
#include <stdio.h>
#include <string.h>

#include "car.h"
#include "cgdk.h"

void help (void)
{
    printf ("Syntax is: CAR command <car_file.ext> <files>\n"
	    "\n"
	    " commands: a - add files to CAR archive\n"
	    "           x - extract files from CAR archive\n"
	    "           l - list the content of the CAR archive\n"
	    "\n");
}

int main (void)
{
  TCAR *car;
  int i;

  printf ("CAR - Cybertek ARchiver\n"
	  "\n%s\n",CGDK_copyright);

  CGDK_init ();

  if (_argc < 2)
  {
    help ();
    return 1;
  }

  if ((strcmpi (_argv [1], "a")) == 0)
  {
    if (_argc < 1+3)
    {
      help ();
      return 1;
    }

    printf ("Creating CAR file - %s\n",_argv [2]);

    if ((CAR_create (_argv [2], _argv [3]))==0)
    {
      printf ("\CAR file created succesfully.\n");
      return 0;
    }
    else
    {
      printf ("\nError creating CAR file.\n");
      return 2;
    }
  }

  if ((strcmpi (_argv [1], "x")) == 0)
  {
    if (_argc < 1+3)
    {
      help ();
      return 1;
    }

    if (CAR_extract (_argv [2], _argv [3]) == 0)
    {
      sprintf (exitStr, "%s/%s extracted succesfully.\n",_argv [2], _argv [3]);
      CGDK_exit (exitStr, EXIT_FAILURE);
    }
    else
    {
      sprintf (exitStr, "Error extracting %s/%s.\n",_argv [2],_argv [3]);
      CGDK_exit (exitStr, EXIT_FAILURE);
    }
  }

  if ((strcmpi (_argv [1], "l")) == 0)
  {
    if (_argc < 1+2)
    {
      help ();
      return 1;
    }

    car = CAR_open (_argv [2]);
    if (car == NULL)
    {
      sprintf (exitStr, "Invalid CAR archive %s.\n");
      CGDK_exit (exitStr, EXIT_FAILURE);
    }

    CAR_loadCacheAll (car);

    CAR_load (car, _argv [3]);

    printf ("File name       Length\n");
    printf ("----------------------\n");
    for (i = 0; i < car->header.filesCount; i++)
    {
      printf ("%-12s%10li\n", car->item [i].name, car->item [i].len);
    }

    CAR_close (car);
  }

  return 0;
}
