
#include <math.h>
#include <stdlib.h>
#include <string.h>

#include "dpmi.h"
#include "plasma.h"


/********** you are recommended to modify only this code **********/
/******************************************************************/

#define P_CALL PLASMA_gen001

#define P_DESCRIPTION "Cybertek plasma image 1.0\n"\
                      "This is default image for 640 x 400, 256c VGA mode. (C) 1996 Cybertek Electronics.\n"
#define P_BYTESPERPIXEL 1
#define P_MOVE_STEPSMAX 10000
#define P_COL_STEPSMAX 10000
#define P_VIDEO_MAXX 640
#define P_VIDEO_MAXY 400
#define P_POOL_MAXX 1.6 * P_VIDEO_MAXX
#define P_POOL_MAXY 1.6 * P_VIDEO_MAXY
#define P_POOL_DIFX (P_POOL_MAXX - P_VIDEO_MAXX)/2
#define P_POOL_DIFY (P_POOL_MAXY - P_VIDEO_MAXY)/2
#define P_COL_MAX 255

/******************************************************************/
/******************************************************************/



void PLASMA_genValues (TPLASMA_image *pi, int basecolor)
{
  float x,y,step;
  long  i;

  /* First generate the plasma map.  This is just an
     function of x and y which gives a smooth surface */

  for (i=0,y=0;y<P_POOL_MAXY;y++)
    for (x=0;x<P_POOL_MAXX;x++,i++)

      /********** you are recommended to modify only this code **********/
      /******************************************************************/
      ((char *) pi->pool) [i] =

      basecolor-1 + ((P_COL_MAX/4)+(P_COL_MAX/25)*
      (sin(x/30) + cos(y/46) +
      cos(x/37) + sin(y/70) +
      sin((x+y)/43) +
      cos(hypot(256-x,150-y)/20)));
      /******************************************************************/
      /******************************************************************/

  /* Then moves for two pointers */

  for (i=0,step=0;step<P_MOVE_STEPSMAX;step++,i+=2)
  {
    /********** you are recommended to modify only this code **********/
    /******************************************************************/

    pi->move [i] =

    P_POOL_DIFX+(P_POOL_DIFX-1)*cos(step/32)+P_POOL_MAXX*
    (long)(P_POOL_DIFY+(P_POOL_DIFY-1)*sin(step/16));


    pi->move [i+1] =

    P_POOL_DIFX+(P_POOL_DIFX-1)*sin(step/21)+P_POOL_MAXX
    *(long)(P_POOL_DIFY+(P_POOL_DIFY-1)*cos(step/24));
    /******************************************************************/
    /******************************************************************/

  }

  /* And a smooth colour table */

  for (step = 0, i = 0; step<P_MOVE_STEPSMAX; step++, i++)

    /********** you are recommended to modify only this code **********/
    /******************************************************************/

    if (i<P_COL_MAX)
    {
      ((long *) pi->col) [i] = 0;
    }
    else
    {
      pi->col [i].r = sin(step/20)*sin(step/15)*124+124;
      pi->col [i].g = sin(step/35)*sin(step/22)*124+124;
      pi->col [i].b = sin(step/13)*sin(step/30)*124+124;
      pi->col [i].a = 0;
    /******************************************************************/
    /******************************************************************/
    }
}




TPLASMA_image *P_CALL (int basecolor)
{
  TPLASMA_image *pi;

  if ((pi = (TPLASMA_image *) _calloc (sizeof (TPLASMA_image), 1)) == NULL)
  {
    return NULL;
  }

  strcpy (pi->desc, P_DESCRIPTION);
  pi->type = FTYPE_CYBERTEK_PLASMA_IMAGE;
  pi->bytesPerPixel = P_BYTESPERPIXEL;
  pi->videoSizeX = P_VIDEO_MAXX;
  pi->videoSizeY = P_VIDEO_MAXY;

  pi->pool = (long *) _alloc (P_POOL_MAXX*P_POOL_MAXY*P_BYTESPERPIXEL);
  if (pi->pool == NULL)
  {
    _free (pi);
    return NULL;
  }
  pi->poolSizeX = P_POOL_MAXX;
  pi->poolSizeY = P_POOL_MAXY;

  pi->move = (long *) _alloc (P_MOVE_STEPSMAX*2*sizeof (long));
  if (pi->move == NULL)
  {
    _free (pi->pool);
    _free (pi);
    return NULL;
  }

  pi->moveStepsMax = P_MOVE_STEPSMAX;
  pi->moveStep = 0;

  pi->col = (TRGBColor *) _alloc (P_COL_STEPSMAX*sizeof (TRGBColor));
  if (pi->col == NULL)
  {
    _free (pi->col);
    _free (pi->pool);
    _free (pi);
    return NULL;
  }

  pi->colBase = basecolor;
  pi->colMax = P_COL_MAX;
  pi->colStepsMax = P_COL_STEPSMAX;
  pi->colStep = 0;

  PLASMA_genValues (pi, basecolor);

  return pi;
}
