
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <mem.h>

#include "svga.h"
#include "cgdk.h"
#include "car.h"
#include "pal.h"
#include "keyb.h"
#include "spr.h"
#include "scroll.h"
#include "mouse.h"
#include "tm.h"
#include "timer.h"

#define DEFAULT_CSF_FILENAME "scifi.csf"



int main (void)
{
  TCAR *carHandle;
  char *textBuf;
  int textSizeX, textSizeY;
  short *mapBuf;
  char *sprBuf;
  TCSF *fnt;
  char fntFileName [80];
  int videoMode;
  char error [256];
  TRGBColor pal [256] = {0};

  printf ("\n"
          "VIEWER - Easy! ASCII text file viewer\n"
          "\n%s\n",CGDK_copyright);

  CGDK_init ();
  TIMER_init (TIMER_OLDFREQ_DEF);

  #ifdef __TURBOC__
    videoMode = XVGA_256x200x256 | DOUBLE_BUFFER;
  #else
    videoMode = XVGA_320x240x256;
  #endif

  switch (_argc-1)
  {
    case 2:
    {
      videoMode = 0;
      videoMode = strtol (_argv [2],NULL,0);
      if (!videoMode)
      {
        sprintf (error, "Invalid video mode %s.\n",_argv [2]);
        CGDK_exit (error, EXIT_FAILURE);
      }
    }
    case 1:
      strcpy (fntFileName, CGDK_getProgramPath (DEFAULT_CSF_FILENAME));
      break;
    default:
    {
      CGDK_exit ("Syntax is : VIEWER <file.txt> [video_mode]\n", EXIT_FAILURE);
    }
  }

  if (SVGA_compatible < VGA_COMPATIBLE)
  {
    CGDK_exit ("This program requires a VGA compatible videocard.\n",1);
  }

  carHandle = CAR_open (CAR_EMULATION);

  fnt = CSF_new (carHandle, fntFileName);
  if (fnt == NULL)
  {
    sprintf (error,"Error reading font file %s.\n",fntFileName);
    CGDK_exit (error, 2);
  }

  CSF_updateColors (fnt, pal, 1, 255);

  if (SVGA_setVideoMode (videoMode) == 0)
  {
    textBuf = CSF_createTextBuf (carHandle, _argv [1]);
    if (textBuf)
    {
      CSF_getTextBufSize (textBuf, &textSizeX, &textSizeY, fnt);

      mapBuf = CSF_createTextMapBuffer (textBuf, textSizeX, textSizeY);
      if (mapBuf)
      {
        sprBuf = CSF_createTextSprBuffer (fnt);
        if (sprBuf)
        {
          KEYB_init ();
          SCROLL_setMap (mapBuf, textSizeX, textSizeY);
          SCROLL_setSpr (sprBuf, fnt->sizeX, fnt->sizeY);
          PAL_clear (0, 256);
          SCROLL_redraw (0, 0, -1, -1);
          PAL_fadeIn (pal, 100, 0, 256);

          SCROLL_walk ();

          PAL_fadeOut (pal, 100, 0, 256);
        }
        else
        {
          CGDK_exit ("Not enough memory.\n",1);
        }
      }
      else
      {
        CGDK_exit ("Not enough memory.\n",1);
      }
    }
    else
    {
      sprintf (error, "Error reading text file %s.\n",_argv [1]);
      CGDK_exit (error, 2);
    }
  }
  else
  {
    CGDK_exit ("Unable to set the specified video mode.\n", EXIT_FAILURE);
  }
  return 0;
}
